#!/usr/bin/python

import atom.http
import urllib
import xml.dom.minidom

class YandexAuth(object):
    def __init__(self, login, password, realm, http_client=None):
        self.http_client = http_client or atom.http.ProxiedHttpClient()
        self.realm = realm

        auth_response = self.http_client.request('GET', 'http://auth.mobile.yandex.ru/yamrsa/key/')
        response_body = auth_response.read()

        if auth_response.status != 200:
            raise RuntimeError('Cannot get RSA key')

        doc = xml.dom.minidom.parseString(response_body)
        key = self.getText(doc.getElementsByTagName('key'))
        request_id = self.getText(doc.getElementsByTagName('request_id'))

        login_str = '<credentials login="%s" password="%s"/>' % (login, password)
        credentials_encrypted = self.encrypt_yarsa(key, login_str)
        token_request = urllib.urlencode({'request_id': request_id, 'credentials': credentials_encrypted})

        auth_response = self.http_client.request('POST', 'http://auth.mobile.yandex.ru/yamrsa/token/', data=token_request, headers={'Content-Type':'application/x-www-form-urlencoded'})
        response_body = auth_response.read()

        doc = xml.dom.minidom.parseString(response_body)
        if auth_response.status != 200:
            raise RuntimeError('Cannot authenticate: ' + self.getText(doc.getElementsByTagName('error')))

        self.token = self.getText(doc.getElementsByTagName('token'))

    def getAuthHeader(self):
        return 'FimpToken realm="%s", token="%s"' % (self.realm, self.token)

    def getAuthToken(self):
        return YandexAuthToken(self.realm, self.token)

    def getText(self, nodelist):
        rc = ""
        for parent in nodelist:
            for node in parent.childNodes:
                if node.nodeType == node.TEXT_NODE:
                    rc = rc + node.data
        return rc

    def encrypt_yarsa(self, key, text):
        NSTR,ESTR = key.split("#")
        DATA_ARR = [ord(x) for x in text]
        N,E,STEP_SIZE = int(NSTR,16),int(ESTR,16), len(NSTR)/2-1

        prev_crypted = [0]*STEP_SIZE

        hex_out = ""
        for i in range(0,(len(DATA_ARR)-1)/STEP_SIZE+1):
            tmp = DATA_ARR[i*STEP_SIZE:(i+1)*STEP_SIZE]
            tmp = [tmp[i] ^ prev_crypted[i] for i in range(0,len(tmp))]
            tmp.reverse()
            plain = 0
            for x in range(0,len(tmp)): plain+= tmp[x]*pow(256, x, N)
            hex_result = "%x" % pow(plain,E,N)
            hex_result = "".join(['0']*( len(NSTR)- len(hex_result))) + hex_result

            for x in range(0,min(len(hex_result),len(prev_crypted)*2),2):
                prev_crypted[x/2] = int(hex_result[x:x+2],16)

            hex_out += ("0" if len(tmp) < 16 else "") + ("%x" % len(tmp)) + "00" # current size
            ks = len(NSTR)/2
            hex_out += ("0" if ks < 16 else "") + ("%x" % ks) + "00" # key size
            hex_out += hex_result

        return hex_out.decode("hex").encode("base64").replace("\n","")


class YandexAuthToken(object):
    def __init__(self, realm, token):
        self.realm = realm
        self.token = token

    def modify_request(self, http_request):
        http_request.headers['Authorization'] = 'FimpToken realm="%s", token="%s"' % (self.realm, self.token)

    ModifyRequest = modify_request
